<?

function install_error_handler($errno, $errstr, $errfile, $errline) {
  global $error;
  if (!$error) {
    switch ($errno) {
      case FATAL:
        echo "<b>FATAL</b> [$errno] $errstr<br />\n";
        echo "  Fatal error in line $errline of file $errfile";
        echo ", PHP " . PHP_VERSION . " (" . PHP_OS . ")<br />\n";
        echo "Aborting...<br />\n";
        
        break;
      case ERROR:
        echo "<b>ERROR</b> [$errno] $errstr<br />\n";
        break;
      case WARNING:
        echo "<b>WARNING</b> [$errno] $errstr<br />\n";
        break;
      }
   }
}

if (DRUPAL_NO_INIT == 1) {
  // Avoid database errors causing a crash
  set_error_handler("install_error_handler");
}


function install_invoke($module,$function) {
  global $db_type;
  $func = sprintf("%s_%s_%s", $module, $function, $db_type);
  if (!function_exists($func)) {
    $func = sprintf("%s_%s",$module,$function);
    if (!function_exists($func)) {
      $func = sprintf("%s",$function);
      if (!function_exists($func)) {
       return false;
      }
    }
  }
  ob_start();
  $func();
  $output = ob_get_contents();
  ob_end_clean();

  return $output;
}


function install_update($module, $start) {
  global $sql_updates;

  $updates = array_slice($sql_updates , ($start-- ? $start : 0));
  $output = "<div style='height : 300px;overflow:auto'>";
  foreach ($updates as $date => $func) {
    #$output .= install_invoke($module,'updates');
    $output .= "<strong>$date</strong><br />\n";
    $ret = $func();
    foreach ($ret as $return) {
     
     $output .= theme_install_query($return[1],$return[2]);
    }
    variable_set($module . "update_start", $date);
  }
  $output .= "</div>";

  return $output;
}

function install_install($module,$edit = "") {
  $output = "<div style='height : 300px;overflow:auto'>";
  $output .= install_invoke($module,"install");
  $output .= "</div>";

  $date = module_invoke($module, "registry", "version");
  variable_set($module . "_install_start", $date);

  return $output;
}

function install_test_database($db_url) {
 $url = parse_url($db_url);
  if ($url['scheme'] == "mysql") {
    include_once "includes/database.mysql.inc";
  }
  else {
    include_once "includes/database.pear.inc";
  }
  $valid = db_connect($db_url);
  if (function_exists('db_close')) {
    db_close();
  }
  return $valid;
}

function install_test_config($db_url, $prefix) {
  $url = parse_url($db_url);
  if ($url['scheme'] == "mysql") {
    include_once "includes/database.mysql.inc";
  }
  else {
    include_once "includes/database.pear.inc";
  }

  global $db_prefix;
  $dp = $db_prefix;
  $db_prefix = $prefix;

  $valid = db_connect($db_url);

  if (is_null(db_query("SELECT * FROM {users}"))) {
    $valid = false;
  }
  if (function_exists('db_close')) {
    db_close();
  }
  $db_prefix = $dp;

  return $valid;
}

function install_create_config($conf,  $db_base,$db_url,$db_prefix, $base_url, $allow_override = 'no', $load = 0) {
  session_destroy();
  $template = '<?php
// $Id: settings.php,v 1.9 2005/02/28 16:42:39 dries Exp $

/**
 * @file
 * Drupal site-specific configuration file.
 *
 * The configuration file which will be loaded is based upon the rules
 * below.
 *
 * The configuration directory will be discovered by stripping the
 * website\'s hostname from left to right and pathname from right to
 * left.  The first configuration file found will be used and any
 * others will be ignored.  If no other configuration file is found
 * then the default configuration file at \'sites/default\' will be used.
 *
 * Example for a ficticious site installed at
 * http://www.drupal.org/mysite/test/ the \'settings.php\' is
 * searched in the following directories:
 *
 *  1. sites/www.drupal.org.mysite.test
 *  2. sites/drupal.org.mysite.test
 *  3. sites/org.mysite.test
 *
 *  4. sites/www.drupal.org.mysite
 *  5. sites/drupal.org.mysite
 *  6. sites/org.mysite
 *
 *  7. sites/www.drupal.org
 *  8. sites/drupal.org
 *  9. sites/org
 *
 * 10. sites/default
 */

/**
 * Database settings:
 *
 *   Note that the $db_url variable gets parsed using PHP\'s built-in
 *   URL parser (i.e. using the "parse_url()" function) so make sure
 *   not to confuse the parser.  In practice, you should avoid using
 *   special characters that are not used in "normal" URLs either.
 *   That is, the use of \':\', \'/\', \'@\', \'?\', \'=\' and \'#\', \'\'\', \'"\',
 *   and so on is likely to confuse the parser; use alpha-numerical
 *   characters instead.
 *
 *   To specify multiple connections to be used in your site (i.e. for
 *   complex custom modules) you can also specify an associative array
 *   of $db_url variables with the \'default\' element used until otherwise
 *   requested.
 *
 *   If an optional $db_prefix is specified all database table names
 *   will be prepended with this string.  Be sure to use valid database
 *   characters only, usually alphanumeric and underscore.  If no
 *   prefixes are desired, set to empty string "".
 *
 *   Database URL format:
 *   $db_url = \'mysql://db_user:db_password@db_hostname/db_name\';
 *   $db_url = \'pgsql://db_user:db_password@db_hostname/db_name\';
 */
$db_url = \'%s\';
$db_prefix = \'%s\';

/**
 * Base URL:
 *
 *   The URL of your website\'s main page.  It is not allowed to have
 *   a trailing slash; Drupal will add it for you.
 */
$base_url = \'%s\';
$allow_override_config = \'%s\';

/**
 * PHP settings:
 *
 *   To see what PHP settings are known to work well, read the PHP
 *   documentation at http://www.php.net/manual/en/ini.php#ini.list
 *   and take a look at the .htaccess file to see which settings are
 *   used there. Settings defined here should not be duplicated there
 *   to avoid conflict issues.
 */
ini_set(\'session.cache_expire\',     200000);
ini_set(\'session.cache_limiter\',    \'none\');
ini_set(\'session.gc_maxlifetime\',   200000);
ini_set(\'session.cookie_lifetime\',  2000000);
#ini_set(\'session.save_handler\',     \'user\');
ini_set(\'session.use_only_cookies\', 1);
ini_set(\'session.use_trans_sid\',    0);

/**
 * Variable overrides:
 *
 *   To override specific entries in the \'variable\' table for this site,
 *   set them here.  You usually don\'t need to use this feature.  This is
 *   useful when used in a configuration file for a vhost or directory,
 *   rather than the default settings.php. Any configuration setting from
 *   the variable table can be given a new value.
 */
// $conf = array(
//   \'site_name\' => \'My Drupal site\',
//   \'theme_default\' => \'pushbutton\',
//   \'anonymous\' => \'Visitor\'
// );
?>';
  
  
  #$contents = sprintf($template, $db_url, $db_prefix, $base_url, 'array("en" => "english")', $allow_override, mktime());
  $contents = sprintf($template, $db_url, $db_prefix, $base_url, $allow_override, mktime());

  $file = fopen($conf,"w");
  fwrite($file,$contents);
  fclose($file);
  if ($load) {
    $sql = "CREATE DATABASE `$db_base` ;";
    db_query($sql);
    return install_db_connect( $db_url,$db_prefix,$base_url = "");
  }
  else {
    return ture;
  }
}

function install_db_connect( $db_url,$db_prefix,$base_url = "") {
  $url = parse_url($db_url);
  if ($url['scheme'] == "mysql") {
    include_once "includes/database.mysql.inc";
  }
  else {
    include_once "includes/database.pear.inc";
  }
  $GLOBALS['db_type'] = $url['scheme'];
  $GLOBALS['db_prefix'] = $db_prefix;
  $GLOBALS['db_url'] = $db_url;
  $GLOBALS['base_url'] = $base_url;
  #$GLOBALS['languages'] = $languages;
  
  return db_connect($db_url);
}



function install_wizards($module) {
  $wizards = module_invoke($module,"registry","configs");
  $edit = $_POST['edit'];
  $finished = ture;
  $wiznum = 0;
  foreach (array_keys($wizards) as $wiz) {
    $wiznum++;
    
    if ($finished) {
      if (($wiz != 'install') && module_invoke($module, "config_$wiz"."_validate",$edit)) {
        $form .= module_invoke($module,"config_$wiz",$edit,1);
      } 
      else {
        $cur_wiz = $wiznum;
        $title = $wizards[$wiz];
        $form .= form_hidden("submitted",$wiz);
        $form .= theme("box", "",  module_invoke($module, "install_help", $wiz));
        if ($edit['submitted'] == $wiz) {
          $form .= install_print_errors();
        }
        if ($wiz == 'install') {
          $ins = install_install($module,$edit);
          if ($edit['debug_sql'] == 1) {
            $form .= $ins;
          }
        } 
        else {
          $form .= module_invoke($module,"config_$wiz",$edit,0);
        }
        $finished = false;
      }
    }
      
  }
  
  $form .= form_hidden("destination",'install');
  if (!$finished && ($cur_wiz != sizeof($wizards))) {
    $form .= form_submit(lt("next"));
  }
  else {
    $form .= module_invoke($module, "install_help", "install_done");
  }

  $output .= form_group($title, form($form));

  return $output; 
}

function install_error($error) {
  global $install_errors;
  $install_errors[] = $error;
}

function install_print_errors($field = "") {
  global $install_errors;


  if (sizeof($install_errors) > 0) {
    foreach ($install_errors as $error) {
      $output .=  theme("error", $error);
    }
  }

  return $output;
}

function install_load_config($conf) {
  include $conf;
  if (isset($db_url)) {
    $url = parse_url($db_url);
    if ($url['scheme'] == "mysql") {
      include_once "includes/database.mysql.inc";
    }
    else {
      include_once "includes/database.inc";
    }
    if (install_test_database($db_url)) {
      if (install_test_config($db_url, $db_prefix)) {
        $GLOBALS['active_config'] = $conf;
        $GLOBALS['allow_override_config'] = $allow_override_config;
        return install_db_connect( $db_url,$db_prefix,$base_url = ""); 
      }
      
    }
  }

  return false;
}

function lt($string,$args = 0){
  global $lang;
  $filename = "modules/install/languege/install.".$lang;
  if(is_file ($filename)){
    $size = filesize($filename);
    $file = fopen($filename,"r");
    while (!feof ($file)) {
      $buffer = fgets($file, $size);
      list ($org_str, $lang2, $data2) = split ("\t", $buffer);
      if($lang == $lang2 && $string == $org_str && !is_null($data2)){
        $string = $data2;
      }
    }
    fclose($file);
  }
  if (!$args) {
    return $string;
  }
  else {
    return strtr($string, $args);
  }
}
?>
